import numpy as np
import argparse
import cv2

# Text parameters.
FONT_FACE = cv2.FONT_HERSHEY_SIMPLEX
FONT_SCALE = 0.5
THICKNESS = 1

# Colors
BLACK  = (0,0,0)
BLUE   = (255,178,50)
YELLOW = (0,255,255)
RED = (0,0,255)

parser = argparse.ArgumentParser(description='Use MobileNet SSD on VF2 for object detection')
parser.add_argument('--device', type=int, default=4, help='camera device, 1 for CSI camera with ISP, 4 for UVC in common case.')
parser.add_argument("--prototxt", default="MobileNetSSD_deploy.prototxt")
parser.add_argument("--weights", default="MobileNetSSD_deploy.caffemodel")
parser.add_argument('--gpu', type=int, default=1, help='device used for inferencing, 1 for GPU, 0 for CPU.')
args = parser.parse_args()

# Labels of the Network.
classNames = { 0: 'background',
    1: 'aeroplane', 2: 'bicycle', 3: 'bird', 4: 'boat',
    5: 'bottle', 6: 'bus', 7: 'car', 8: 'cat', 9: 'chair',
    10: 'cow', 11: 'diningtable', 12: 'dog', 13: 'horse',
    14: 'motorbike', 15: 'person', 16: 'pottedplant',
    17: 'sheep', 18: 'sofa', 19: 'train', 20: 'tvmonitor' }

def cal_position(detections,i,cols,rows):
    # Class_id
    class_id = int(detections[0, 0, i, 1])

    # Calculate the positions
    xLeftBottom = int(detections[0, 0, i, 3] * cols)
    yLeftBottom = int(detections[0, 0, i, 4] * rows)
    xRightTop = int(detections[0, 0, i, 5] * cols)
    yRightTop = int(detections[0, 0, i, 6] * rows)

    heightFactor = frame.shape[0] / 300.0
    widthFactor = frame.shape[1] / 300.0

    xLeftBottom = int(widthFactor * xLeftBottom)
    yLeftBottom = int(heightFactor * yLeftBottom)
    xRightTop = int(widthFactor * xRightTop)
    yRightTop = int(heightFactor * yRightTop)

    return class_id, xLeftBottom, yLeftBottom, xRightTop, yRightTop

def do_detect(frame, net, classNames):

    # Resize the frame to 300x300
    frame_resized = cv2.resize(frame, (300, 300))

    # Put image into the network and pridict the results
    blob = cv2.dnn.blobFromImage(frame_resized, 0.007843, (300, 300), (127.5, 127.5, 127.5), False)
    net.setInput(blob)
    detections = net.forward()

    # Put efficiency information. The function getPerfProfile returns the overall time for inference(t) and the timings
    # for each of the layers(in layersTimes)
    t, _ = net.getPerfProfile()
    label = 'Inference time: %.2f ms' % (t * 1000.0 / cv2.getTickFrequency())
    cv2.putText(frame, label, (20, 40), FONT_FACE, FONT_SCALE, RED, THICKNESS, cv2.LINE_AA)

    cols = frame_resized.shape[1]
    rows = frame_resized.shape[0]

    for i in range(detections.shape[2]):
        # Get the value of confidence
        confidence = detections[0, 0, i, 2]
        # The bounding boxes with confidences less than or equal to 0.5 will be eliminated
        if confidence > 0.5:

            # Get information about class and position of the objects
            class_id, xLeftBottom, yLeftBottom, xRightTop, yRightTop = cal_position(detections,i, cols,rows)
            #cv2.rectangle(frame, (xLeftBottom, yLeftBottom), (xRightTop, yRightTop),  (255, 0, 0), 2)
            cv2.rectangle(frame, (xLeftBottom, yLeftBottom), (xRightTop, yRightTop), BLUE, 2*THICKNESS)
            if class_id in classNames:
                label = classNames[class_id]
                text_size = cv2.getTextSize(label, FONT_FACE, FONT_SCALE, THICKNESS)
                dim, baseline = text_size[0], text_size[1]
                cv2.rectangle(frame, (xLeftBottom, yLeftBottom), (xLeftBottom + dim[0], yLeftBottom + dim[1] + baseline), BLACK, cv2.FILLED)
                cv2.putText(frame, label, (xLeftBottom, yLeftBottom + dim[1]), FONT_FACE, FONT_SCALE, YELLOW, THICKNESS, cv2.LINE_AA)

    return  frame

# Open selected camera using selected API
deviceID = args.device
apiID = cv2.CAP_V4L2
cap = cv2.VideoCapture()
cap.open(deviceID, apiPreference=apiID)

if deviceID == 1:
    cap.set(cv2.CAP_PROP_CONVERT_RGB, 0)

# Load model
net = cv2.dnn.readNetFromCaffe(args.prototxt, args.weights)

# To set the selected banckend and target
if args.gpu:
   net.setPreferableBackend(cv2.dnn.DNN_BACKEND_OPENCV)
   net.setPreferableTarget(cv2.dnn.DNN_TARGET_OPENCL)

while True:
    # Capture frames
    ret, frame = cap.read()
    if deviceID == 1:
        frame = cv2.cvtColor(frame, cv2.COLOR_YUV2BGR_NV12)
    # Process
    frame = do_detect(frame,net,classNames)
    # Display the results
    cv2.imshow("MobileNet-SSD", frame)

    if cv2.waitKey(1) >= 0:
        break
