import cv2
import numpy as np
import argparse

# PATHS
YOLOV3_WEIGHTS = "/usr/share/opencv4/yolo-v3/yolov3-tiny.weights"
YOLOV3_CONFIG = "/usr/share/opencv4/yolo-v3/yolov3-tiny.cfg"
YOLOV3_CLASSES = "/usr/share/opencv4/yolo-v3/classes.txt"

# DNN Constants.
INPUT_WIDTH = 416
INPUT_HEIGHT = 416
CONFIDENCE_THRESHOLD = 0.5
NMS_THRESHOLD = 0.4

# Text parameters.
FONT_FACE = cv2.FONT_HERSHEY_SIMPLEX
FONT_SCALE = 0.7
THICKNESS = 2

def pre_process(input_image, net):
    blob = cv2.dnn.blobFromImage(input_image, 0.00392, (INPUT_WIDTH, INPUT_HEIGHT), (0, 0, 0), True, crop=False)
                
    net.setInput(blob)

    output_layers = net.getUnconnectedOutLayersNames()

    outputs = net.forward(output_layers)
    return outputs

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('--device', type=int, default=4, help='camera device number, 1 for CSI Camera with ISP, 4 for UVC in common case.')
    args = parser.parse_args()
    deviceID = args.device
    apiID = cv2.CAP_V4L2

    # open selected camera using selected API
    cap = cv2.VideoCapture()
    cap.open(deviceID, apiPreference=apiID)
    if deviceID == 1:
        cap.set(cv2.CAP_PROP_CONVERT_RGB, 0)

    # Load Yolo
    net = cv2.dnn.readNet(YOLOV3_WEIGHTS, YOLOV3_CONFIG)
    net.setPreferableBackend(cv2.dnn.DNN_BACKEND_OPENCV)
    net.setPreferableTarget(cv2.dnn.DNN_TARGET_OPENCL)
    classes = []
    with open(YOLOV3_CLASSES, "r") as f:
        classes = [line.strip() for line in f.readlines()]
    layer_names = net.getLayerNames()

    colors = np.random.uniform(0, 255, size=(len(classes), 3))
   
    while True:
        ret, frame = cap.read()
        if deviceID == 1:
            frame = cv2.cvtColor(frame, cv2.COLOR_YUV2BGR_NV12)
        height, width = frame.shape[:2]

        # Pre-processing: Detecting objects
        outs = pre_process(frame, net)

        # Post-processing: Showing informations on the screen
        class_ids = []
        confidences = []
        boxes = []
        for out in outs:
            for detection in out:
                scores = detection[5:]
                class_id = np.argmax(scores)
                confidence = scores[class_id]
                if confidence > 0.5:
                    # Object detected
                    center_x = int(detection[0] * width)
                    center_y = int(detection[1] * height)
                    w = int(detection[2] * width)
                    h = int(detection[3] * height)

                    # Rectangle coordinates
                    x = int(center_x - w / 2)
                    y = int(center_y - h / 2)

                    boxes.append([x, y, w, h])
                    confidences.append(float(confidence))
                    class_ids.append(class_id)

        indexes = cv2.dnn.NMSBoxes(boxes, confidences, CONFIDENCE_THRESHOLD, NMS_THRESHOLD)

        for i in range(len(boxes)):
            if i in indexes:
                x, y, w, h = boxes[i]
                label = str(classes[class_ids[i]])
                color = colors[i]
                cv2.rectangle(frame, (x, y), (x + w, y + h), color, THICKNESS)
                cv2.putText(frame, label, (x, y - 20), FONT_FACE, FONT_SCALE, color, THICKNESS)

        # Put efficiency information. The function getPerfProfile returns the overall time for inference(t) and the timings for each of the layers(in layersTimes)
        t, _ = net.getPerfProfile()
        label = 'Inference time: %.2f ms' % (t * 1000.0 / cv2.getTickFrequency())
        cv2.putText(frame, label, (20, 40), FONT_FACE, FONT_SCALE, (255,255,0), THICKNESS)
        cv2.namedWindow("Image", 0)
        cv2.resizeWindow("Image", 1280, 960)
        cv2.imshow("Image", frame)

        # Press 'q' to quit
        if cv2.waitKey(1) == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()
